% Computes the ANM series for a SYSTENSEUR object
% at the given point (vector), using the given tangent vector.

% Using the "tensor" formalism:
%
%     Q is a 3-tensor, here stored as a row vector of neq matrices
% Q_tens = [Q1 Q2 Q3 ... Qneq] where each "Qi" is a ninc by ninc matrix.
%

% Note that:
%  neq is the number of equations in the L0/L/Q part
%  ninc = neq + 1

function [Ups,FpnlA] = calculeseries(systens,Uo,Utj)

global Q_tens

%% Matrice tangente systeme au point de depart
dRdU = get_dRdU(systens,Uo);

%% Vecteur tangent exact a la branche au point de depart
U1 = tangente(systens,dRdU,Uo);


if (U1'*Utj) <0
  U1 = - U1;
end

%% Matrice tangente de la MAN
K = [dRdU ; (get_chemin(systens).*U1)'];
%invK = inv(K);
[LK,UK,pK,qK] = lu(K,'vector'); % fast LU-decomposition

[neq, ninc] = size(dRdU);

pmax = get_ordre(systens); % ANM series max. order
Ups = []; % ANM series coefficients: [Uo U1 ... Up ... Upmax]


%% 
% Point de départ (current solution point)
% 
U0=sparse(Uo);
Ups = U0; % Note that Ups(:,p+1) contains Up

%%                 
% Premier Ordre  (current tangent) 
%                 

Ups = [Ups, U1];

%%                 
% Ordres suivants 
%                 

for p=2:pmax

% Calcul du second membre à l'ordre p (computes the series coeff.)
%   
%   The calculation is done iteratively : the p_th order coeff. Up is deduced
%   from K*Up = Fpnl, where Fpnl depends only on the previous orders
%   and K is the tangent matrix of the problem that we LU-decomposed at the
%   beginning

Fpnl = zeros(ninc,1) ; % will contain the RHS

% Main calculus:
%       Fpnl = sum_{r=1}^{p-1}  -Q(Ur,U_{p-r}) 

% This is the standard way of dealing with the Q tensor coded as horizontal
% concatenated matrices

%%% [version sans dQdU : plus lent mais demande moins de mémoire]
%%% 25/07/2011  S.K.

for r=1:p-1
    Fpnl = Fpnl - [ Ups(:,p-r+1)'*reshape(Ups(:,r+1)'*Q_tens,ninc,neq), 0 ]' ;
end


Up(qK) = UK\(LK\Fpnl(pK)) ; % fast inversion using LU factorisation
Ups = [Ups, Up'] ;

% Here, for p=2 we get Ups = [U0, U1, U2]
end


%% Second membre supplementaire (additional right-hand side)
FpnlA = zeros(neq,1);

%       FpnlA = sum_{r=1}^{p}  -Q(Ur,U_{p+1-r})

for r=1:p
    FpnlA = FpnlA - [ Ups(:,p+1-r+1)'*reshape(Ups(:,r+1)'*Q_tens,ninc,neq) ]' ;
end

disp(['branch sign : ',num2str(sign(prod(diag(UK))))]);
