% detectbifurc   Bifurcation detection.
%    [Bbif,stab]=detectbifurc(Nds,Us,Ustab) returns:
%    + in matrix Bbif:
%      - the indices of the bifurcation points in the first row 
%      - the type of bifurcation in the second row :
%         * 1 saddle-node, pitchfork, or transcritical
%         * 2 Hopf bifurcation
%
%    + in vector stab (with length(stab)=size(U,2)) a 1 if the
%      branch is unstable and a 0 if the branch is stable. It is the
%      same as Ustab(1,:)
%
%    Us is the matrix whose columns are vectors of unknowns of the final 
%    algebraic system solved by Manlab
% 
%    Ustab contains the stability data
%
%    Nds is the initial dynamical system size (this is also the number
%    of Floquet multipliers)
%
%    By Sami Karkar (after O. Thomas in detectbifurcHBM.m)
%      - initial version: Aug. 2011
%   Last revision : June 2014 (S. Karkar)


function [Bbif,stab]=detectbifurc(Nds,Us,Ustab)


if size(Ustab,1)==1
  stab=ones(size(Us(1,:)));
  Bbif=NaN;
  return
end

global ML_tolstab

tolstab=ML_tolstab;
disp(['Stability computation tolerance: tolstab = ',num2str(tolstab)])

Npoint=size(Ustab,2);
lambda = Us(end,:);

ALP=Ustab(2:(Nds+1),:); % Floquet exponents "alpha"
ALP=sort(ALP);

% EIGinst: matrix of same size that ALP, that contains 1 if
% unstable and 0 if stable
EIGinst=zeros(size(ALP)); 
I=find(real(ALP)>tolstab);
EIGinst(I)=ones(size(EIGinst(I)));
% Neiginst: row vector that contains the number of Floquet
% exponent to the right of the imaginary axis
Neiginst=sum(EIGinst,1); 
% stab: row vector that is 1 if it is unstable and zero if it is stable
stab=ones(1,size(ALP,2));
I=find(Neiginst>0);
stab(I)=zeros(size(stab(I)));
%  0 1 1 1 1 1 1 2  2 1  1 0 0 2 2 0 0   -> Neiginst(1:end-1)
%0 1 1 1 1 1 1 2 2  1 1  0 0 2 2 0 0     -> Neiginst(2:end)
%  1 0 0 0 0 0 1 0 -1 0 -1 0 2 0 2 0     -> Neigbif
Neigbif=Neiginst(2:end)-Neiginst(1:end-1); % Contains the number of
                                           % Floquet multipliers
                                           % that cross the imaginary axis
Ibif=find(Neigbif~=0);      % Indices where there is a bifurcation
Tbif=zeros(1,length(Ibif)); % row vector of size the number of
                            % bifurcation points

for ii=1:length(Ibif); 
  if abs(Neigbif(Ibif(ii)))==2
    if Neigbif(Ibif(ii))>0
        alfasbif=ALP(:,Ibif(ii)+1);
    else
        alfasbif=ALP(:,Ibif(ii));
    end
    ialfasinst=find(real(alfasbif)>tolstab);
    alfasinst=alfasbif(ialfasinst);
    [realfabif,ialfabif]=min(real(alfasinst));
    imagalfabif=imag(alfasinst(ialfabif));
    if imagalfabif~=0
        disp(['lambda = ',num2str(lambda(Ibif(ii))),' - Hopf bifurcation with f = ' num2str(abs(imagalfabif)/(2*pi))])
        Tbif(ii)=2;
    else
        disp(['lambda = ',num2str(lambda(Ibif(ii))),' - Double Zero bifurcation']);
        Tbif(ii)=4;
    end
  elseif abs(Neigbif(Ibif(ii)))==1
    disp(['lambda = ',num2str(lambda(Ibif(ii))),' - Bifurcation (Saddle-node, Pitch-fork, or Transcritical)'])
    Tbif(ii)=1;
  elseif abs(Neigbif(Ibif(ii)))>2
    disp(['lambda = ',num2str(lambda(Ibif(ii))),' - Bifurcation with co-dimension > 1'])
    Tbif(ii)=3;
  end
end

Bbif=[Ibif;Tbif];