% detectbifurcHBM   Bifurcation detection.
%    [Bbif,stab]=detectbifurcHBM(Nds,U,Ustab) returns:
%    + in matrix Bbif:
%      - the indices of the bifurcation points in the first row 
%      - the type of bifurcation in the second row :
%         * 1 for a simple bifurcation (saddle-node, pitchfork...)
%         * 2 for a period doubling bifurcation
%         * 3 for Neimark-Sacker bifurcation
%    + in vector stab (with length(stab)=size(U,2)) a 1 if the
%      branch is unstable and a 0 if the branch is stable. It is the
%      same as Ustab(1,:)
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab
% 
%    Ustab contains the stability data
%
%    Nds is the initial dynamical system size (this is also the number
%    of Floquet multipliers)
%
%    By O. Thomas 
%      - initial version: Sept. 2010
%   Modified May 2011 (bug fixed: detection of NS bifurcation
%      when two Floquet multipliers enter the unit circle. The abs
%      in the test abs(neigbif)==2 was missing)
%   Last revision : June 2014 (S. Karkar)

function [Bbif,stab]=detectbifurcHBM(Nds,U,Ustab)


if size(Ustab,1)==1
  stab=ones(size(U(1,:)));
  Bbif=NaN;
  return
end

global ML_tolstab

tolstab=ML_tolstab;
disp(['Stability computation tolerance: tolstab = ',num2str(tolstab)])

Npoint=size(Ustab,2);
omega = U(end,:);

ALP=Ustab(2:(Nds+1),:); % Floquet exponents
ALP=sort(ALP);
T=2*pi./omega;          % Period
TT=ones(Nds,1)*T;
RHO=exp(ALP.*TT);       % Floquet multipliers
RHO=sort(RHO,1);       

% EIGinst: matrix of same size that ALP/RHO, that contains 1 if
% unstable and 0 if stable
EIGinst=zeros(size(ALP)); 
I=find(abs(RHO)>(1+tolstab));
EIGinst(I)=ones(size(EIGinst(I)));
% Neiginst: row vector that contains the number of Floquet
% multipliers outside the unit circle
Neiginst=sum(EIGinst,1); 
% stab: row vector that is 1 if it is unstable and zero if it is stable
stab=ones(1,size(ALP,2));
I=find(Neiginst>0);
stab(I)=zeros(size(stab(I)));
%  0 1 1 1 1 1 1 2  2 1  1 0 0 2 2 0 0   -> Neiginst(1:end-1)
%0 1 1 1 1 1 1 2 2  1 1  0 0 2 2 0 0     -> Neiginst(2:end)
%  1 0 0 0 0 0 1 0 -1 0 -1 0 2 0 2 0     -> Neigbif
Neigbif=Neiginst(2:end)-Neiginst(1:end-1); % Contains the number of
                                           % Floquet multipliers
                                           % that cross the unit circle
Ibif=find(Neigbif~=0);      % Indices where there is a bifurcation
Tbif=zeros(1,length(Ibif)); % row vector of size the number of
                            % bifurcation points

for ii=1:length(Ibif); 
  ibif=Ibif(ii);
  neigbif=Neigbif(ibif);
%  test=EIGinst(:,ibif)-EIGinst(:,ibif+1);
  if abs(neigbif)==2
    disp(['lambda = ',num2str(omega(ibif)),' - Neimark-Sacker bifurcation'])
    Tbif(ii)=3;
  else
    Ieiginst_bef=find(EIGinst(:,ibif)==1);     % row indices of
                                               % unstable RHO before
                                               % the bifurcation point
    Ieiginst_aft=find(EIGinst(:,ibif+1)==1);   % rhow indices of
                                               % unstable RHO after
                                               % the bifurcation point
    remod_bef=real(RHO(Ieiginst_bef,ibif));    % Real parts of these RHO before...
    remod_aft=real(RHO(Ieiginst_aft,ibif+1));  % Real parts of these RHO after...
    Npos_bef=length(find(remod_bef>0));        % Number of RHO with
                                               % positive real part before...
    Npos_aft=length(find(remod_aft>0));        % Number of RHO with
                                               % positive real part after...
    if Npos_bef~=Npos_aft % change of the number of rho outside the
                          % unit circle with positive real part
      disp(['lambda = ',num2str(omega(ibif)),' - Simple bifurcation'])
      Tbif(ii)=1;
    else
      disp(['lambda = ',num2str(omega(ibif)),' - Period doubling bifurcation'])
      Tbif(ii)=2;
    end
  end
end

Bbif=[Ibif;Tbif];