% plotbranchXHBM   Branch plot for Manlab/HBM with user defined x axis
% 
%    Hand=plotbranchXHBM(H,Neq,U,Ustab,X,Idisp,Hdisp,fig,[xlab]) plots the 
%    amplitude of the harmonics components of the solution computed by the
%    Manlab/HBM method as a function of vector X.
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab and Ustab the stability data. 
% 
%    X is a vector, with  length(X)=size(U,2)
%
%    Idisp contains the indices of the entries of the initial physical 
%    unknown vector u(t) to be displayed and Hdisp the corresponding
%    harmonics number.
%
%    H is the number of harmonics of the simulation.
%
%    Neq is the number of equations of the initial system of unknown
%    u(t)
% 
%    fig is a figure/axis handle to plot the bargraph on.
%
%    (optional) [xlab] is a string containing the name of the user defined X
%    axis
%
%    It returns in Hand a column vector of handles to lineseries objects, 
%    one handle per plotted line.
%
%    Example: Hand=plotbranchXHBM(obj.H,obj.Neq,U,Ustab,PERIOD,[1 1 1],[1 3 5],1)
%             plots the three first odd harmonics (H=1, 3, 5) of u1
%             as a function of the PERIOD vector 
%
%    By O. Thomas / Oct. 2010
%%%  Last revision : S. Karkar / June 2014

function Hand=plotbranchXHBM(H,Neq,U,Ustab,X,Idisp,Hdisp,fig)

if ~isempty(varargin)
    xlab=varargin{1};
else
    xlab='lambda';
end

pencolorML;
style = {':','-'};

% Activate axis
% =============

if floor(fig)==fig
  figure(fig)
  hold on
else
  axes(fig)
  hold on
end

% Plotting the branches
% =====================

%  0 1 1 1 1 1  1 0 0 0 0 0 0
%0 1 1 1 1 1 1  0 0 0 0 0 0
%  1 0 0 0 0 0 -1 0 0 0 0 0

Nb=length(Idisp);
Hand=[];
Handleg=[];
hold on

omega = U(end,:);
if size(Ustab,1)>1
  stab = Ustab(1,:);
else
  stab=ones(size(omega));
end
Ibif=find((stab(2:end)-stab(1:end-1))~=0);
Ibif=[1 Ibif size(U,2)];


for ib=1:Nb
  idisp=Idisp(ib);
  hdisp=Hdisp(ib);
  colloc=Col(ib,:);
  if hdisp==0
    I0=idisp;
    Amp=U(I0,:);
  else
    Icos=Neq+idisp+(hdisp-1)*2*Neq;
    Amp=sqrt(U(Icos,:).^2+U(Icos+Neq,:).^2);
  end
  for bb=1:(length(Ibif)-1)
    II=Ibif(bb):Ibif(bb+1);
    st=style{stab(Ibif(bb)+1)+1};
    hand=plot(X(II),Amp(II),st,'Color',colloc,'linewidth',1);
    Hand=[Hand;hand];
    if stab(Ibif(bb)+1)==1
      Handleg(ib)=hand;
    end
  end
  if length(Handleg)<ib
    Handleg(ib)=hand;
  end
  plot(X(Ibif(2:end-1)),Amp(Ibif(2:end-1)),'.','Color',colloc)
  Leg{ib}=['H_',num2str(hdisp),'u_',num2str(idisp)];
end
legend(Handleg,Leg,'location','best')
box on
ylabel('Harmonics amplitude')
xlabel(xlab)

  
