% plotbranchbif   Branch plot for Manlab/Fixed-Points.
%    Hand=plotbranchbif(Neq,Nds,Us,Ustab,Idisp,fig) plots the 
%    amplitude of the harmonics components of the solution computed by the
%    Manlab/Fixed-point method as a function of the continuation parameter
%    lambda.
%    It also displays the bifurcation points:
%    - B for a simple bifurcation (saddle-node, pitchfork...)
%    - H for period doubling bifurcation
%
%
%    Us is the matrix whose columns are vectors of unknowns of the 
%    algebraic system solved by Manlab and Ustab the stability data. 
% 
%    Idisp contains the indices of the entries of the unknow vector
%    to be displayed.
%
%    Neq is the number of equations of the system
%
%    Nds is the initial dynamical system size (this is also the number
%    of Floquet multipliers). It is the total number of unknown minus one
%    (lambda) minus the number of additional variables.
%
%    fig is a figure/axis handle to plot the bargraph on.
%
%    It returns in Hand a column vector of handles to lineseries objects, 
%    one handle per plotted line.
%
%    Example: Hand=plotbranchbif(,obj.Neq,2,Us,Ustab,[1 1 1],1)
%             plots the branch in the plane (lambda,u1). 
%
%    By Sami Karkar (after O. Thomas in plotbranchbifHBM.m)
%       - initial version : Aug. 2011

function Hand=plotbranchbif(Neq,Nds,Us,Ustab,Idisp,fig)

pencolorML;
style = {':','-'};

% Activate axis
% =============

if floor(fig)==fig
  figure(fig)
  hold on
else
  axes(fig)
  hold on
end

% Plotting the branches
% =====================

%  0 1 1 1 1 1  1 0 0 0 0 0 0
%0 1 1 1 1 1 1  0 0 0 0 0 0
%  1 0 0 0 0 0 -1 0 0 0 0 0

Nb=length(Idisp);
Hand=[];
Handleg=[];
hold on

lambda = Us(end,:);
%stab = Ustab(1,:);  
[Bbif,stab]=detectbifurc(Nds,Us,Ustab);
Ibif=[1 Bbif(1,:) size(Us,2)];
Tbif=Bbif(2,:);

for ib=1:Nb
  idisp=Idisp(ib);
  colloc=Col(ib,:);
  Amp=Us(idisp,:);
  for bb=1:(length(Ibif)-1)
    II=Ibif(bb):Ibif(bb+1);
    st=style{stab(Ibif(bb)+1)+1};
    hand=plot(lambda(II),Amp(II),st,'Color',colloc,'linewidth',1);
    Hand=[Hand;hand];
    if stab(Ibif(bb)+1)==1
      Handleg(ib)=hand;
    end
  end
  if length(Handleg)<ib
    Handleg(ib)=hand;
  end
  plot(lambda(Ibif(2:end-1)),Amp(Ibif(2:end-1)),'+','Color',colloc)
  Leg{ib}=['U_{',num2str(idisp),'}'];
  for ib=1:size(Bbif,2)
    ibif=Bbif(1,ib);
    tbif=Bbif(2,ib);
    switch tbif
     case 1
      str='B';
     case 2
      str='H';
     case 3
      str='B*';
     case 4
      str='ZZ';
    end
    text(lambda(ibif),Amp(ibif),str,'margin',3,'verticalalignment','Bottom')
  end
end
legend(Handleg,Leg,'location','best')
box on
ylabel('Component amplitude')
xlabel('\lambda')
  
  
return
