% plotdspHBM   DSP graph for Manlab/HBM (dB scale)
%    Hand = plotdspHBM(H,Neq,U,Idisp,fig) plots a graph of the DSP
%    the solution computed by the Manlab/HBM method.
%    It displays the amplitudes of the harmonic components in dB scale.
%
%    U is the vector of unknowns of the final algebraic system solved
%    by Manlab. 
% 
%    Idisp contains the indices of the entries of the initial physical 
%    unknown vector u(t) to be displayed.
%
%    H is the number of harmonics of the simulation.
%
%    Neq is the number of equations of the initial system of unknown
%    u(t)
% 
%    fig is a figure/axis handle to plot the bargraph on.
%
%    It returns a vector of barseries handles in Hand. Each entry of
%    Hand correspond to a bar
%
%    Example: Hand=plotbarHBM(obj.H,obj.Neq,U,[1 2],1)
%
%    By S. Karkar / Sept 2011
%       (inspired by the function "plotbarHBM" by O. Thomas / Sept. 2010)

function Hand=plotdspHBM(H,Neq,U,Idisp,fig)

pencolorML;

% Column vector test
% ==================

if size(U,2)>size(U,1)
  U=U';
end

if size(U,2)>1
  U=U(:,1);
  disp('tracbarU: U should be a column vector')
end

% Plotting the bargraph
% =====================

if floor(fig)==fig
  figure(fig)
  clf
  hold on
else
  axes(fig)
  hold on
end

Utrac=[];
Hand=[];
Ndisp=length(Idisp);

for iu=1:Ndisp
  idisp=Idisp(iu);
  colloc=Col(iu,:);
  Icos=Neq+(idisp:2*Neq:((H-1)*2*Neq+idisp));
  Isin=2*Neq+(idisp:2*Neq:((H-1)*2*Neq+idisp));
  Uamp=sqrt(U(Icos).^2+U(Isin).^2);
  Uamp=[abs(U(idisp));Uamp];
  hand=plot((0:H),20*log10(Uamp),'x--');
  set(hand,'Color',colloc)
  Hand=[Hand;hand];
  Leg{iu}=['u_{',num2str(idisp) '}'];
end

% Labels
% ======

Xtick=(0:H);
Ntick=length(Xtick);
Xtickstr=cell(1,Ntick);
Xtickstr{1}='H0';
for ii=2:Ntick
  Xtickstr{ii}=['H',num2str(ii-1)];
end

dI=ceil(H/10);
if dI>1
  Xtickstr0=cell(1,Ntick);
  Xtickstr0(2:dI:end)=Xtickstr(2:dI:end);
  Xtickstr=Xtickstr0;
end

set(gca,'Xtick',Xtick,'XtickLabel',Xtickstr,'Xlim',[0 H])
ylabel('Harmonics amplitude (dB)')
box on

legend(Hand,Leg);


